var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { Tokens } from "../tokens.js";
import { ERC20, MAX_APPROVAL_AMOUNT } from "../bridge/erc20.js";
import { useSignerFromEthereum } from "./signer.js";
import { logError, parseApproveAmount } from "./helpers.js";
import { AllowanceError, TransactionError } from "./errors.js";
import { useEffect, useState } from "react";
var TX_STATUS_SUCCESS = 1;
function useCheckAllowance(ethereum, chainId) {
    var getSigner = useSignerFromEthereum()[0];
    var _a = useState(null), allowance = _a[0], setAllowance = _a[1];
    function fn(args) {
        getSigner(ethereum).getAddress()
            .then(function (ownerAddress) {
            var token = args.token, spender = args.spender;
            var tokenArgs = { tokenAddress: token.address(chainId), chainId: chainId };
            ERC20.allowanceOf(ownerAddress, spender, tokenArgs)
                .then(function (res) { return setAllowance(res); })
                .catch(function (e) {
                var err = e instanceof Error ? e : new Error(e);
                var allowanceErr = new AllowanceError(ownerAddress, spender, token, "exception thrown from ERC20.allowanceOf()", err);
                console.error(allowanceErr);
            });
        });
    }
    return [fn, allowance];
}
function useApproveTokenSpend(ethereum, chainId) {
    var getSigner = useSignerFromEthereum()[0];
    var _a = useState(null), tx = _a[0], setTx = _a[1];
    function fn(args) {
        Tokens.approveTokenSpend(__assign(__assign({}, args), { amount: parseApproveAmount(args.amount, args.token, chainId), chainId: chainId, signer: getSigner(ethereum) }))
            .then(setTx)
            .catch(logError);
    }
    return [fn, tx];
}
function useApproveStatus(ethereum, chainId) {
    var _a = useCheckAllowance(ethereum, chainId), checkAllowance = _a[0], allowance = _a[1];
    var _b = useState(false), approveComplete = _b[0], setApproveComplete = _b[1];
    var _c = useState(null), wantApproveAmount = _c[0], setWantApproveAmount = _c[1];
    function fn(args) {
        var token = args.token, spender = args.spender, amount = args.amount, approveTx = args.approveTx;
        var txHash = approveTx.hash;
        var wantAmt = amount ? parseApproveAmount(amount, token, chainId) : MAX_APPROVAL_AMOUNT;
        approveTx.wait(1)
            .then(function (txResult) {
            var txStatus = txResult.status;
            if (txStatus) {
                if (txStatus !== TX_STATUS_SUCCESS) {
                    var txErr = new TransactionError(txHash, "reverted");
                    console.error(txErr);
                    return;
                }
            }
            Promise.resolve(checkAllowance({ spender: spender, token: token })).then(function () { return setWantApproveAmount(wantAmt); });
        })
            .catch(function (e) {
            var err = e instanceof Error ? e : new Error(e);
            var txErr = new TransactionError(txHash, "exception thrown from .wait(1)", err);
            console.error(txErr);
        });
    }
    useEffect(function () {
        if (allowance && wantApproveAmount) {
            var checkAmt = wantApproveAmount;
            if (wantApproveAmount.eq(MAX_APPROVAL_AMOUNT)) {
                checkAmt = MAX_APPROVAL_AMOUNT.sub(5);
            }
            if (allowance.gte(checkAmt)) {
                setApproveComplete(true);
            }
        }
    }, [allowance, wantApproveAmount]);
    return [fn, approveComplete, allowance];
}
export { TransactionError, AllowanceError, useApproveTokenSpend, useApproveStatus, useCheckAllowance };
